#    Back In Time
#    Copyright (C) 2008-2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import os.path
import pygtk
pygtk.require("2.0")
import gtk
import gnomevfs


class GnomeFileIcons:
	def __init__( self ):
		self.all_icons = gtk.icon_theme_get_default().list_icons()
		self.cache = {}
		self.user_path = os.path.expanduser( '~' )
		self.special_folders = {}

		self.update_special_folder_icons()

	def add_special_folder_( self, path, icon_name ):
		if len( path ) <= 0:
			return

		if not icon_name in self.all_icons:
			icon_name = 'folder'

		self.special_folders[ path ] = icon_name

	def get_special_folder_path( self, name ):
		path = ''

		try:
			pipe = os.popen( "cat %s/.config/user-dirs.dirs 2>/dev/null | grep %s=" % ( self.user_path, name ), 'r' )
			path = pipe.read()
			pipe.close()
		except:
			pass

		if len( path ) <= 0:
			return ''

		path = path[ len( name ) + 1 : ]
		if path[ 0 ] == '"':
			path = path[ 1 : -2 ]

		path = path.replace( '$HOME', self.user_path )
		return path

	def update_special_folder_icons( self ):
		self.special_folders = {}

		#add desktop
		self.add_special_folder_( self.get_special_folder_path( 'XDG_DESKTOP_DIR' ), 'user-desktop' )
		
		#add home
		self.add_special_folder_( self.user_path, 'user-home' )
		
	def get_icon( self, path ):
		if not os.path.exists(path):
			return gtk.STOCK_FILE

		#check if it is a special folder
		if path in self.special_folders:
			return self.special_folders[path]

		#get mime type
		mime_type = gnomevfs.get_mime_type( gnomevfs.escape_path_string( path ) )
		mime_type = mime_type.replace( '/', '-' )

		#search in the cache
		if mime_type in self.cache:
			return self.cache[mime_type]

		#print "path: " + path
		#print "mime: " + mime_type

		#try gnome mime
		items = mime_type.split('-')
		for aux in xrange(len(items)-1):
			icon_name = "gnome-mime-" + '-'.join(items[:len(items)-aux])
			if icon_name in self.all_icons:
				#print "icon: " + icon_name
				self.cache[mime_type] = icon_name
				return icon_name

		#try folder
		if os.path.isdir( path ):
			icon_name = 'folder'
			if icon_name in self.all_icons:
				#print "icon: " + icon_name
				self.cache[mime_type] = icon_name
				return icon_name

			#print "icon: " + icon_name
			icon_name = gtk.STOCK_DIRECTORY
			self.cache[mime_type] = icon_name
			return icon_name

		#try simple mime
		for aux in xrange(len(items)-1):
			icon_name = '-'.join(items[:len(items)-aux])
			if icon_name in self.all_icons:
				#print "icon: " + icon_name
				self.cache[mime_type] = icon_name
				return icon_name

		#file icon
		icon_name = gtk.STOCK_FILE
		self.cache[mime_type] = icon_name
		return icon_name




#    Back In Time
#    Copyright (C) 2008-2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os.path
import os


class ConfigFile:
	def __init__( self ):
		self.dict = {}

	def save( self, filename ):
		try:
			file = open( filename, 'w' )
			keys = self.dict.keys()
			keys.sort()
			for key in keys:
				file.write( "%s=%s\n" % ( key, self.dict[key] ) )
			file.close()
		except:
			pass

	def load( self, filename ):
		self.dict = {}
		self.append( filename )

	def append( self, filename ):
		lines = []

		try:
			file = open( filename, 'r' )
			lines = file.readlines()
			file.close()
		except:
			pass

		for line in lines:
			items = line.split( '=' )
			if len( items ) == 2:
				self.dict[ items[ 0 ] ] = items[ 1 ][ : -1 ]

	def remap_key( self, old_key, new_key ):
		if old_key != new_key:
			if self.dict.has_key( old_key ):
				if not self.dict.has_key( new_key ):
					self.dict[ new_key ] = self.dict[ old_key ]
				del self.dict[ old_key ]

	def has_value( self, key ):
		return self.dict.has_key( key )

	def get_str_value( self, key, default_value = '' ):
		if self.dict.has_key( key ):
			return self.dict[ key ]
		else:
			return default_value

	def set_str_value( self, key, value ):
		self.dict[ key ] = value

	def get_int_value( self, key, default_value = 0 ):
		try:
			return int( self.dict[ key ] )
		except:
			return default_value

	def set_int_value( self, key, value ):
		self.set_str_value( key, str( value ) )

	def get_bool_value( self, key, default_value = False ):
		try:
			val = self.dict[ key ]
			if "1" == val or "TRUE" == val.upper():
				return True
			return False
		except:
			return default_value

	def set_bool_value( self, key, value ):
		if value:
			self.set_str_value( key, 'true' )
		else:
			self.set_str_value( key, 'false' )

	def remove_key( self, key ):
		if self.dict.has_key( key ):
			del self.dict[ key ]

	def remove_keys_starts_with( self, prefix ):
		remove_keys = []

		for key in self.dict.iterkeys():
			if key.startswith( prefix ):
				remove_keys.append( key )

		for key in remove_keys:
			del self.dict[ key ]


class ConfigFileWithProfiles( ConfigFile ):
	def __init__( self, default_profile_name ):
		ConfigFile.__init__( self )

		self.default_profile_name = default_profile_name
		self.current_profile_id = '0'

	def _get_profile_key_( self, key, profile_id = None ):
		if profile_id is None:
			profile_id = self.current_profile_id
		return 'profile.' + profile_id + '.' + key

	def remove_profile_key( self, key, profile_id = None ):
		self.remove_key( self._get_profile_key_( key, profile_id ) )

	def remove_profile_keys_starts_with( self, prefix, profile_id = None ):
		self.remove_keys_starts_with( self._get_profile_key_( prefix, profile_id ) )

	def has_profile_value( self, key, profile_id = None ):
		return self.dict.has_key( self._get_profile_key_( key, profile_id ) )

	def get_profile_str_value( self, key, default_value = '', profile_id = None ):
		return self.get_str_value( self._get_profile_key_( key, profile_id ), default_value )

	def set_profile_str_value( self, key, value, profile_id = None ):
		self.set_str_value( self._get_profile_key_( key, profile_id ), value )

	def get_profile_int_value( self, key, default_value = 0, profile_id = None ):
		return self.get_int_value( self._get_profile_key_( key, profile_id ), default_value )

	def set_profile_int_value( self, key, value, profile_id = None ):
		self.set_int_value( self._get_profile_key_( key, profile_id ), value )

	def get_profile_bool_value( self, key, default_value = False, profile_id = None ):
		return self.get_bool_value( self._get_profile_key_( key, profile_id ), default_value )

	def set_profile_bool_value( self, key, value, profile_id = None ):
		self.set_bool_value( self._get_profile_key_( key, profile_id ), value )

	def get_current_profile( self ):
		return self.current_profile_id

	def get_profiles( self ):
		return self.get_str_value( 'profiles', '0' ).split(':')

	def get_profile_name( self, profile_id = None ):
		return self.get_profile_str_value( 'profile_name', self.default_profile_name, profile_id )

	def add_profile( self, name ):
		profiles = self.get_profiles()

		for profile in profiles:
			if self.get_profile_name( profile ) == name:
				return _('Profile %s already exists !') % name

		new_id = 1
		while True:
			ok = True

			for profile in profiles:
				if profile == str(new_id):
					ok = False
					break

			if ok:
				break

			new_id = new_id + 1

		new_id = str( new_id )

		self.profiles.append( new_id )
		self.set_str_value( 'profiles', profiles.join(':') )

		self.current_profile_id = new_id
		self.set_profile_str_value( 'profile_name', name, new_id )
		return None

	def remove_profile( self, profile_id = None ):
		if profile_id == None:
			profile_id = self.current_profile_id

		if profile_id == '0':
			return

		profiles = self.get_profiles()
		index = 0
		for profile in profiles:
			if profile == profile_id:
				self.remove_keys_starts_with( self._get_profile_key_( '', profile_id ) )
				del profiles[index]
				self.set_str_value( 'profiles', profiles.join(':') )
				break
			index = index + 1

		if self.current_profile == profile_id:
			self.current_profile = '0'

	def rename_profile( self, name, profile_id = None ):
		if profile_id == None:
			profile_id = self.current_profile_id

		if profile_id == 0:
			return _('You can\'t rename "Main" profile')

		profiles = self.get_profiles()

		for profile in profiles:
			if self.get_profile_name( profile ) == name:
				if profile[0] == profile_id:
					return None #
				return _('Profile %s already exists !') % name

		return None




#    Back In Time
#    Copyright (C) 2008-2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import os.path
import pluginmanager
import tools
import logger


class UserCallbackPlugin( pluginmanager.Plugin ):
	def __init__( self ):
		return

	def init( self, snapshots ):
		self.config = snapshots.config
		self.callback = self.config.get_take_snapshot_user_callback()
		if not os.path.exists( self.callback ):
			return False
		return True

	def notify_callback( self, args = '' ):
		logger.info( "[UserCallbackPlugin.notify_callback] %s" % args )
		os.system( "sh \"%s\" %s" % ( self.callback, args ) )

	def on_process_begins( self ):
		self.notify_callback( '1' )

	def on_process_ends( self ):
		self.notify_callback( '2' )

	def on_error( self, code, message ):
		if len( message ) <= 0:
			self.notify_callback( "4 %s" % code )
		else:
			self.notify_callback( "4 %s \"%s\"" % ( code, message ) )

	def on_new_snapshot( self, snapshot_id, snapshot_path ):
		self.notify_callback( "3 %s \"%s\"" % ( snapshot_id, snapshot_path ) )




#    Back In Time
#    Copyright (C) 2008-2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os.path
import os
import configfile
import sys
import tools


class DriveInfo( configfile.ConfigFile ):
	def __init__( self, path ):
		configfile.ConfigFile.__init__( self )

		self.path = path
		self.load( self._get_driveinfo_file_() )

		dirty = False

		if sys.platform == 'win32':
			#there is nothing to do
			pass
		else:
			if not self.has_value( 'hardlinks' ):
				self.set_bool_value( 'hardlinks', self._check_hardlinks_() )
				dirty = True

			if not self.has_value( 'permissions' ):
				self.set_bool_value( 'permissions', self._check_perms_() )
				dirty = True

			if not self.has_value( 'usergroup' ):
				self.set_bool_value( 'usergroup', self._check_usergroup_() )
				dirty = True

		if dirty:
			self.save( self._get_driveinfo_file_() )

	def support_hardlinks( self ):
		return self.get_bool_value( 'hardlinks', False )
		
	def support_permissions( self ):
		return self.get_bool_value( 'permissions', False )
		
	def support_usergroup( self ):
		return self.get_bool_value( 'usergroup', False )
		
	def _get_driveinfo_file_( self ):
		return os.path.join( self.path, 'driveinfo' )

	def _check_hardlinks_( self ):
		tmp_path = os.path.join( self.path, 'driveinfo.tmp' )
		tools.make_dirs( tmp_path )
		if not os.path.isdir( tmp_path ):
			return False

		file1_path = os.path.join( tmp_path, 'file1' )
		file2_path = os.path.join( tmp_path, 'file2' )

		ret_val = False

		os.system( "echo abc > \"%s\"" % file1_path )
		os.system( "ln \"%s\" \"%s\"" % ( file1_path, file2_path ) )
		os.system( "echo abc > \"%s\"" % file2_path )

		if os.path.exists( file1_path ) and os.path.exists( file2_path ):
			try:
				info1 = os.stat( file1_path )
				info2 = os.stat( file2_path )

				if info1.st_size == info2.st_size:
					ret_val = True
			except:
				pass

		os.system( "rm -rf \"%s\"" % tmp_path )
		return ret_val

	def _check_perms_for_file_( self, file_path, mode ):
		ret_val = False

		os.system( "chmod %s \"%s\"" % ( mode, file_path ) )
		try:
			info = "%o" % os.stat( file_path ).st_mode
			info = info[ -3 : ]
			if info == mode:
				ret_val = True
		except:
			pass

		return ret_val

	def _check_perms_( self ):
		tmp_path = os.path.join( self.path, 'driveinfo.tmp' )
		tools.make_dirs( tmp_path )
		if not os.path.isdir( tmp_path ):
			return False

		file_path = os.path.join( tmp_path, 'file' )
		os.system( "echo abc > \"%s\"" % file_path )
		if not os.path.isfile( file_path ):
			return False

		ret_val = False

		if self._check_perms_for_file_( file_path, '111' ):
			if self._check_perms_for_file_( file_path, '700' ):
				if self._check_perms_for_file_( file_path, '600' ):
					if self._check_perms_for_file_( file_path, '711' ):
						if self._check_perms_for_file_( file_path, '300' ):
							if self._check_perms_for_file_( file_path, '666' ):
								ret_val = True

		os.system( "rm -rf \"%s\"" % tmp_path )
		return ret_val

	def _check_usergroup_( self ):
		tmp_path = os.path.join( self.path, 'driveinfo.tmp' )
		tools.make_dirs( tmp_path )
		if not os.path.isdir( tmp_path ):
			return False

		file_path = os.path.join( tmp_path, 'file' )
		os.system( "echo abc > \"%s\"" % file_path )
		if not os.path.isfile( file_path ):
			return False

		ret_val = False

		uid = os.getuid()
		gid = os.getgid()

		try:
			info = os.stat( file_path )
			if info.st_uid == uid and info.st_gid == gid:
				ret_val = True
		except:
			pass

		if ret_val and uid == 0:
			#try to change the group
			import grp

			#search for another group
			new_gid = gid
			new_name = ''
			for group in grp.getgrall():
				if group.gr_gid != gid:
					new_gid = group.gr_gid
					new_name = group.gr_name
					break

			if new_gid != gid:
				os.system( "chgrp %s \"%s\"" % ( new_name, file_path ) )
				try:
					info = os.stat( file_path )
					if info.st_gid != new_gid:
						ret_val = False
				except:
					ret_val = False

		os.system( "rm -rf \"%s\"" % tmp_path )
		return ret_val

